from AbstractService import AbstractService
import Registry
from Version import Version
from Service import ServiceState
from Service import ServiceKind
from datatypes.ImageInput import ImageInput
from datatypes.Command import Command
from datatypes.AiResult import AiResult
from decimal import Decimal

class FlowerAIServiceClientInterface(AbstractService):
    """Partially implemented service interface for "Flower AI Service Client"
       Generated by: EASy-Producer."""
    
    ingestor = None
    param_aiId = "FL-AI"
    
    def __init__(self):
        """Initializes the service.""" 
        super().__init__("FlowerAiServiceClient", "Flower AI Service Client", Version("0.1.0"), "", True, ServiceKind.TRANSFORMATION_SERVICE)
        Registry.services['FlowerAiServiceClient'] = self
        self.registerMethods()
        
    def attachIngestor(self, ingestor):
        """Called by ServiceEnvironment to attach an individual type-generic ingestor, i.e., a function 
           that allows to asynchronously ingest result data back into stream processing. The ingestor
           cares for the type and the target stream.
           
           Parameters:
             - ingestor -- the ingestor function
           """ 
        self.ingestor = ingestor

    def ingest(self, data):
        """Ingests result data asynchronously into stream processing. Target stream and data type are 
           handled by the ingestor.
           
           Parameters:
             - data -- the data to ingest; if the type is unknown/no type translator registered, nothing will happen
           """ 
        if self.ingestor is not None:
            self.ingestor(data)

    def setState(self, state:ServiceState):
        """Changes the state. [R133c]
        
        Parameters:
          - newState -- the new state (ServiceState)
        """
        super().setState(state)
        if (state==ServiceState.STARTING):
            self.start()
        if (state==ServiceState.STOPPING):
            self.stop()

    def start(self):
        """Called when the server shall start.
        """
        pass

    def stop(self):
        """Called when the server shall stop.
        """
        pass

    def notifyReconfigured(self, values: dict):
        """Called to inform the implementing class that a parameter reconfiguration happened.
           
           Parameters:
             - values -- the new param names/values
           """ 
        pass
    
    def reconfigure(self, values:dict):
        """Called by ServiceEnvironment to initialize service parameters or to reconfigure them at runtime.
           
           Parameters:
             - values -- the new param names/values
           """ 
        if "aiId" in values:
            self.param_aiId = values["aiId"]
        self.notifyReconfigured(values)
        
    def getAiId(self):
        """Returns the value of parameter aiId. May change at runtime upon reconfiguration.
           
           Result:
             - the value of the parameter
           """ 
    
        return self.param_aiId
    
    
    def processImageInput(self, data: ImageInput):
        """Asynchronous data processing method. Use self.ingest(data) to pass the result back to the data stream.
           Possible output types: AiResultImpl
    
        Parameters:
          - data -- the data to process
        """
        raise NotImplementedError
    def processCommand(self, data: Command):
        """Asynchronous data processing method. Use self.ingest(data) to pass the result back to the data stream.
           Possible output types: AiResultImpl
    
        Parameters:
          - data -- the data to process
        """
        raise NotImplementedError            
    def registerMethods(self):
        Registry.asyncTransformers['FlowerAiServiceClient_ImageInput'] = self.processImageInput
        Registry.asyncTransformers['FlowerAiServiceClient_Command'] = self.processCommand
