from AbstractService import AbstractService
import Registry
from Version import Version
from Service import ServiceState
from Service import ServiceKind
from datatypes.MipMqttOutput import MipMqttOutput
from datatypes.MipAiPythonOutput import MipAiPythonOutput
from datatypes.MipMqttInput import MipMqttInput
from decimal import Decimal

class MipAiPythonServiceInterface(AbstractService):
    """Partially implemented service interface for "Mip Ai Python Service"
       Generated by: EASy-Producer."""
    
    ingestor = None            
    def __init__(self):
        """Initializes the service.""" 
        super().__init__("MipAiPythonService", "Mip Ai Python Service", Version("0.1.0"), "", True, ServiceKind.TRANSFORMATION_SERVICE)
        Registry.services['MipAiPythonService'] = self
        self.registerMethods()
        
    def attachIngestor(self, ingestor):
        """Called by ServiceEnvironment to attach an individual type-generic ingestor, i.e., a function 
           that allows to asynchronously ingest result data back into stream processing. The ingestor
           cares for the type and the target stream.
           
           Parameters:
             - ingestor -- the ingestor function
           """ 
        self.ingestor = ingestor

    def ingest(self, data):
        """Ingests result data asynchronously into stream processing. Target stream and data type are 
           handled by the ingestor.
           
           Parameters:
             - data -- the data to ingest; if the type is unknown/no type translator registered, nothing will happen
           """ 
        if self.ingestor is not None:
            self.ingestor(data)

    def setState(self, state:ServiceState):
        """Changes the state. [R133c]
        
        Parameters:
          - newState -- the new state (ServiceState)
        """
        super().setState(state)
        if (state==ServiceState.STARTING):
            self.start()
        if (state==ServiceState.STOPPING):
            self.stop()

    def start(self):
        """Called when the server shall start.
        """
        pass

    def stop(self):
        """Called when the server shall stop.
        """
        pass

    
    def processMipMqttOutput(self, data: MipMqttOutput):
        """Asynchronous data processing method. Use self.ingest(data) to pass the result back to the data stream.
           Possible output types: MipAiPythonOutputImpl, MipMqttInputImpl
    
        Parameters:
          - data -- the data to process
        """
        raise NotImplementedError            
    def registerMethods(self):
        Registry.asyncTransformers['MipAiPythonService_MipMqttOutput'] = self.processMipMqttOutput
